from youtube_autonomous.elements.builder.element_builder import ElementBuilder
from youtube_autonomous.elements.validator.element_parameter_validator import ElementParameterValidator
from youtube_autonomous.segments.enums import SegmentField
from yta_image.size import resize_scaling as resize_image_scaling
from yta_multimedia.video.utils import generate_video_from_image
from yta_general_utils.downloader.image import download_image
from yta_general_utils.file.filename import get_file_extension
from yta_general_utils.temp import Temp
from typing import Union


class ImageElementBuilder(ElementBuilder):
    """
    The builder that generates 'image' content for
    the project.
    """

    @classmethod
    def build_from_enhancement(
        cls,
        enhancement: dict
    ):
        # TOOD: Here, what about duration or 'calculated_duration' (?)
        return (
            cls.build_from_filename(enhancement.filename, enhancement.duration)
            if enhancement.filename else
            cls.build_from_url(enhancement.url, enhancement.duration)
        )

    @classmethod
    def build_from_segment(
        cls,
        segment: dict
    ):
        # TOOD: Here, what about duration or 'calculated_duration' (?)
        return (
            cls.build_from_filename(segment.filename, segment.duration)
            if segment.filename else
            cls.build_from_url(segment.url, segment.duration)
        )

    @classmethod
    def build_from_filename(
        cls,
        filename: str,
        duration: Union[float, int]
    ):
        ElementParameterValidator.validate_filename(filename)
        ElementParameterValidator.validate_duration(duration)

        return cls.image_to_video(filename, duration, None)

    @classmethod
    def build_from_url(
        cls,
        url: str,
        duration: Union[float, int]
    ):
        ElementParameterValidator.validate_url(url)
        ElementParameterValidator.validate_duration(duration)

        # TODO: Try to do all this process in memory, writting not the image
        image_filename = download_image(url)

        return cls.image_to_video(image_filename.filename, duration, None)
    
    @classmethod
    def build_from_filename_from_segment(
        cls,
        segment: dict
    ):
        filename = segment.get(SegmentField.FILENAME.value, None)
        duration = segment.get(SegmentField.DURATION.value, None)

        return cls.build_from_filename(filename, duration)

    @classmethod
    def build_from_url_from_segment(
        cls,
        segment: dict
    ):
        url = segment.get(SegmentField.URL.value, None)
        duration = segment.get(SegmentField.DURATION.value, None)

        return cls.build_from_url(url, duration)
    
    # TODO: Move this to a helper (?)
    @classmethod
    def image_to_video(
        cls,
        filename: str,
        duration: Union[float, int],
        effect = None
    ):
        ElementParameterValidator.validate_filename(filename)
        ElementParameterValidator.validate_duration(duration)
        
        # Resize image to fit the screen
        tmp_image_filename = Temp.create_filename(f'image{get_file_extension(filename)}')
        resize_image_scaling(filename, 1920, 1080, tmp_image_filename)

        # TODO: Apply an Effect (this need work)
        return generate_video_from_image(tmp_image_filename, duration)